@@ -1035,6 +1035,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFail)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+  if (BlobIsSeekable(image) && GetBlobSize(image) < 107)
+    ThrowReaderException(CorruptImageError,InsufficientImageDataInFile,image);
   /*
     Initialize structures.
   */
@@ -2002,6 +2004,9 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
     }
   image=imagev;  /* Use 'image' after this point for optimization */
 
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+     "  Write JPEG Image: image->orientation = %d",image->orientation);
+
   /*
     Transform image to user-requested colorspace.
   */
