@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2015 GraphicsMagick Group
+% Copyright (C) 2003-2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -794,12 +794,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
 
   int
     c,
-    code;
+    version;
+
+  unsigned int
+    code,
+    flags;
 
   long
-    flags,
     j,
-    version,
     y;
 
   PICTRectangle
@@ -897,9 +899,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
     if ((version == 1) || ((TellBlob(image) % 2) != 0))
-      code=ReadBlobByte(image);
+      {
+        if ((c=ReadBlobByte(image)) == EOF) /* returns int */
+          break;
+        code=(unsigned int) c;
+      }
     if (version == 2)
-      code=ReadBlobMSBShort(image);
+      code=ReadBlobMSBShort(image); /* returns magick_uint16_t */
     if (code > 0xa1)
       {
         if (IsEventLogging())
@@ -1232,8 +1238,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               if ((code == 0x9a) || (code == 0x9b) ||
                   (bytes_per_line & 0x8000))
                 (void) CompositeImage(image,CopyCompositeOp,tile_image,
-                   destination.left,destination.top);
-                DestroyImage(tile_image);
+                                      destination.left,destination.top);
+            DestroyImage(tile_image);
             if (destination.bottom != (long) image->rows)
               if (!MagickMonitorFormatted(destination.bottom,image->rows,&image->exception,
                                           LoadImageText,image->filename,
@@ -1361,6 +1367,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         tile_image=ReadImage(clone_info,exception);
         (void) LiberateTemporaryFile(clone_info->filename);
         DestroyImageInfo(clone_info);
+        clone_info=(ImageInfo *) NULL;
         if (tile_image == (Image *) NULL)
           continue;
         FormatString(geometry,"%lux%lu",Max(image->columns,tile_image->columns),
@@ -1568,7 +1575,9 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     x;
 
   size_t
-    count;
+    bytes_per_line,
+    count,
+    row_bytes;
 
   unsigned char
     *buffer = (unsigned char *) NULL,
@@ -1579,12 +1588,10 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     status;
 
   unsigned long
-    bytes_per_line,
     storage_class;
 
   unsigned short
     base_address,
-    row_bytes,
     transfer_mode;
 
   /*
@@ -1612,7 +1619,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   source_rectangle=size_rectangle;
   destination_rectangle=size_rectangle;
   base_address=0xff;
-  row_bytes=(unsigned short) (image->columns | 0x8000);
+  row_bytes=image->columns;
   bounds.top=0;
   bounds.left=0;
   bounds.bottom=(short) image->rows;
@@ -1640,7 +1647,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
       pixmap.bits_per_pixel=32;
       pixmap.pack_type=0x04;
       transfer_mode=0x40;
-      row_bytes=(unsigned short) ((4*image->columns) | 0x8000);
+      row_bytes=4*image->columns;
     }
   /*
     Allocate memory.
@@ -1812,7 +1819,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobMSBShort(image,PictPixmapOp);
       (void) WriteBlobMSBLong(image,(unsigned long) base_address);
     }
-  (void) WriteBlobMSBShort(image,row_bytes | 0x8000);
+  (void) WriteBlobMSBShort(image,(unsigned short) (row_bytes | 0x8000));
   (void) WriteBlobMSBShort(image,bounds.top);
   (void) WriteBlobMSBShort(image,bounds.left);
   (void) WriteBlobMSBShort(image,bounds.bottom);
