@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2015 GraphicsMagick Group
+% Copyright (C) 2003 - 2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -327,6 +327,31 @@ static MagickPassFail load_tile (Image* image, Image* tile_image, XCFDocInfo* in
   unsigned char
     *graydata;
 
+  /*
+    Validate that claimed data length is sufficent for tile.
+  */
+  {
+    size_t
+      expected_data_length=0;
+
+    if (inDocInfo->image_type == GIMP_GRAY)
+      {
+        expected_data_length=tile_image->columns*tile_image->rows*
+          sizeof(unsigned char);
+      }
+    else if (inDocInfo->image_type == GIMP_RGB)
+      {
+        expected_data_length=tile_image->columns*tile_image->rows*
+          sizeof(XCFPixelPacket);
+      }
+    if (expected_data_length && (expected_data_length > data_length))
+      {
+        ThrowException(&image->exception,CorruptImageError,CorruptImage,
+                       "Claimed tile data length is insufficient for tile data");
+        return MagickFail;
+      }
+  }
+
   xcfdata = xcfodata = MagickAllocateMemory(XCFPixelPacket *,data_length);
   graydata = (unsigned char *) xcfdata;  /* used by gray and indexed */
 
@@ -734,7 +759,8 @@ static MagickPassFail load_level (Image* image,
         }
       
       /* seek to the tile offset */
-      (void) SeekBlob(image, offset, SEEK_SET);
+      if (SeekBlob(image, offset, SEEK_SET) != offset)
+        ThrowBinaryException(CorruptImageError,InsufficientImageDataInFile,image->filename);
 
       /* allocate the image for the tile 
          NOTE: the last tile in a row or column may not be a full tile!
@@ -880,6 +906,7 @@ static MagickPassFail load_level (Image* image,
 #endif
         }
       DestroyImage(tile_image);
+      tile_image = (Image *) NULL;
 
       /* adjust tile position */
       destLeft++;
@@ -971,7 +998,8 @@ static MagickPassFail load_hierarchy (Image *image, XCFDocInfo* inDocInfo, XCFLa
     ThrowBinaryException(BlobError,UnableToObtainOffset,image->filename);
   
   /* seek to the level offset */
-  (void) SeekBlob(image, offset, SEEK_SET);
+  if (SeekBlob(image, offset, SEEK_SET) != offset)
+    ThrowBinaryException(CorruptImageError,InsufficientImageDataInFile,image->filename);
   
   /* read in the level */
   if (load_level (image, inDocInfo, inLayer) == MagickFail)
@@ -980,7 +1008,8 @@ static MagickPassFail load_hierarchy (Image *image, XCFDocInfo* inDocInfo, XCFLa
   /* restore the saved position so we'll be ready to
    *  read the next offset.
    */
-  (void) SeekBlob(image, saved_pos, SEEK_SET);
+  if (SeekBlob(image, saved_pos, SEEK_SET) != saved_pos)
+    ThrowBinaryException(BlobError,UnableToSeekToOffset,image->filename);
 
   return MagickPass;
 }
@@ -1017,12 +1046,18 @@ static MagickPassFail ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLaye
                           (unsigned long) outLayer->height,
                           (unsigned long) outLayer->type);
 
+  if (EOFBlob(image))
+    ThrowBinaryException(CorruptImageError,InsufficientImageDataInFile,image->filename);
+
   /* allocate the image for this layer */
   outLayer->image=CloneImage(image,outLayer->width, outLayer->height,True,
                              &image->exception);
   if (outLayer->image == (Image *) NULL)
     return MagickFail;
 
+  if (CheckImagePixelLimits(outLayer->image, &image->exception) != MagickPass)
+    ThrowBinaryException(ResourceLimitError,ImagePixelLimitExceeded,image->filename);
+
   /* read the layer properties! */
   foundPropEnd = 0;
   while ( !foundPropEnd && !EOFBlob(image) )
@@ -1118,7 +1153,8 @@ static MagickPassFail ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLaye
     return MagickFail;
 
   /* clear the image based on the layer opacity */
-  (void) SetImage(outLayer->image,(Quantum)(255-outLayer->opacity));
+  if (SetImage(outLayer->image,(Quantum)(255-outLayer->opacity)) != MagickPass)
+    return MagickFail;
 
   /* set the compositing mode */
   outLayer->image->compose = GIMPBlendModeToCompositeOperator( outLayer->mode );
@@ -1133,14 +1169,16 @@ static MagickPassFail ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLaye
   layer_mask_offset = ReadBlobMSBLong(image);
 
   /* read in the hierarchy */
-  (void) SeekBlob(image, hierarchy_offset, SEEK_SET);
+  if (SeekBlob(image, hierarchy_offset, SEEK_SET) != (magick_off_t) hierarchy_offset)
+    ThrowBinaryException(CorruptImageError,InsufficientImageDataInFile,image->filename);
   if (load_hierarchy (image, inDocInfo, outLayer) == MagickFail)
     return MagickFail;
 
   /* read in the layer mask */
   if (layer_mask_offset != 0)
     {
-      (void) SeekBlob(image, layer_mask_offset, SEEK_SET);
+      if (SeekBlob(image, layer_mask_offset, SEEK_SET) != (magick_off_t) layer_mask_offset)
+        ThrowBinaryException(CorruptImageError,InsufficientImageDataInFile,image->filename);
 
 #if 0  /* BOGUS: support layer masks! */
       layer_mask = xcf_load_layer_mask (info, gimage);
@@ -1204,6 +1242,24 @@ static MagickPassFail ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLaye
 %
 %
 */
+#define DestroyLayerInfo(number_layers,layer_info) \
+  do {                                                         \
+    size_t                                                     \
+      j;                                                       \
+                                                               \
+    if (layer_info != (XCFLayerInfo *) NULL)                   \
+      {                                                        \
+        for (j=0; j < number_layers; j++)       \
+          {                                                    \
+            if (layer_info[j].image != (Image *) NULL)         \
+              {                                                \
+                DestroyImage(layer_info[j].image);             \
+                layer_info[j].image = (Image *) NULL;          \
+              }                                                \
+          }                                                    \
+      }                                                        \
+    MagickFreeMemory(layer_info);                              \
+  } while (0);
 static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
@@ -1265,6 +1321,9 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                            (image_type == GIMP_GRAY ? "GRAY" :
                             (image_type == GIMP_INDEXED ? "INDEXED" :
                              "unknown"))));
+
+  if ((image->columns == 0) || (image->rows == 0))
+    ThrowReaderException(CorruptImageError,NegativeOrZeroImageSize,image);
   
   /* setup some things about the image...*/
   image->compression=NoCompression;
@@ -1281,6 +1340,10 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       ThrowReaderException(CoderError,ColormapTypeNotSupported,image);
     }
+  else
+    {
+      ThrowReaderException(CorruptImageError,ImageTypeNotSupported,image);
+    }
   /*
     SetImage can be very expensive and it is not clear that this one is
     actually needed so comment it out for now.
@@ -1380,7 +1443,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       
             /* BOGUS: we don't write these yet because we aren't
-                  reading them properly yet :( */
+               reading them properly yet :( */
             /* image->x_resolution = xres; */
             /* image->y_resolution = yres; */
           }
@@ -1497,8 +1560,8 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         last_layer = 0,
         T = 0;
 
-	MagickBool
-          foundAllLayers = MagickFalse;
+      MagickBool
+        foundAllLayers = MagickFalse;
 
       /* BIG HACK
          because XCF doesn't include the layer count, and we
@@ -1519,7 +1582,9 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           else
             number_layers++;
         } while ( !foundAllLayers );
-      (void) SeekBlob(image, oldPos, SEEK_SET); /* restore the position! */
+
+      if (SeekBlob(image, oldPos, SEEK_SET) != oldPos) /* restore the position! */
+        ThrowReaderException(BlobError,UnableToSeekToOffset,image);
 
       first_layer = image_info->subimage;
       num_layers = number_layers;
@@ -1580,21 +1645,35 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               ThrowReaderException(BlobError,UnableToObtainOffset,image);
             }
 
-          if( first_layer <= current_layer && current_layer <= last_layer )
+          if ( first_layer <= current_layer && current_layer <= last_layer )
 	    {
 	      /* seek to the layer offset */
-	      (void) SeekBlob(image, offset, SEEK_SET);
+	      if (SeekBlob(image, offset, SEEK_SET) != offset)
+                {
+                  /* FIXME: CID 64064: leaks layer_info */
+                  DestroyLayerInfo(number_layers,layer_info);
+                  ThrowReaderException(CorruptImageError,InsufficientImageDataInFile,image);
+                }
 
 	      /* read in the layer */
 	      layer_ok = ReadOneLayer( image, &doc_info, &layer_info[current_layer-first_layer] );
 	      if (!layer_ok)
 		{
+#if 0
 		  int
 		    j;
 
-		  for (j=0; j < (current_layer-first_layer); j++)
-		    DestroyImage(layer_info[j].image);
+		  for (j=0; j <= (current_layer-first_layer); j++)
+                    {
+                      if (layer_info[j].image)
+                        {
+                          DestroyImage(layer_info[j].image);
+                          layer_info[j].image = (Image *) NULL;
+                        }
+                    }
 		  MagickFreeMemory(layer_info);
+#endif
+                  DestroyLayerInfo(number_layers,layer_info);
 		  CopyException(exception,&image->exception);
 		  CloseBlob(image);
 		  DestroyImageList(image);
@@ -1603,7 +1682,12 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 	      /* restore the saved position so we'll be ready to
 	       *  read the next offset.
 	       */
-	      (void) SeekBlob(image, saved_pos, SEEK_SET);
+	      if (SeekBlob(image, saved_pos, SEEK_SET) != saved_pos)
+                {
+                  /* FIXME: CID 64064: leaks layer_info */
+                  DestroyLayerInfo(number_layers,layer_info);
+                  ThrowReaderException(BlobError,UnableToSeekToOffset,image);
+                }
 	    }
 
           current_layer++;
@@ -1615,6 +1699,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 	  (void) CompositeImage(image, OverCompositeOp, layer_info[0].image,
 				layer_info[0].offset_x, layer_info[0].offset_y );
 	  DestroyImage( layer_info[0].image );
+          layer_info[0].image = (Image *) NULL;
 	}
       else
 	{
@@ -1630,6 +1715,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 		CompositeImage(image, OverCompositeOp, layer_info[j].image,
 			       layer_info[j].offset_x, layer_info[j].offset_y );
 		DestroyImage( layer_info[j].image );
+                layer_info[j].image = (Image *) NULL;
 	      }
 	    }
 	  }
@@ -1644,6 +1730,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 				  layer_info[number_layers-1].offset_x,
 				  layer_info[number_layers-1].offset_y );
 	    DestroyImage( layer_info[number_layers-1].image );
+            layer_info[number_layers-1].image = (Image *) NULL;
 
 	    /* now reverse the order of the layers as they are put
 	       into subimages
