@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2015 GraphicsMagick Group
+% Copyright (C) 2003 - 2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -73,21 +73,15 @@ typedef struct _OutputInfo
 
 typedef struct _LogInfo
 {
-  char
-    *path,
-    *filename;
-
-  unsigned long
-    generations,
-    limit;
-
-  char
-    *format;
-
   FILE
     *file;
 
+  TimerInfo
+    timer;
+
   unsigned long
+    generations,
+    limit,
     generation,
     count;
 
@@ -97,11 +91,14 @@ typedef struct _LogInfo
   LogOutputType
     output_type;
 
-  TimerInfo
-    timer;
-
   LogMethod
-    method;
+    method;  /* Logger callback function */
+
+  char
+    path[256],
+    filename[256],
+    format[200];
+
 } LogInfo;
 
 typedef struct _EventInfo
@@ -176,7 +173,7 @@ static volatile unsigned int
   log_configured = False;
 
 static LogInfo
-  *log_info = (LogInfo *) NULL;
+  log_info;
 
 static SemaphoreInfo
   *log_semaphore = (SemaphoreInfo *) NULL;
@@ -244,21 +241,13 @@ MagickExport void DestroyLogInfo(void)
 
   LockSemaphoreInfo(log_semaphore);
 
-  if (log_info != (LogInfo *) NULL)
-    {
-      if (log_info->file != (FILE *) NULL)
-        if ((log_info->file != stdout) && (log_info->file != stderr))
-          {
-            (void) fprintf(log_info->file,"</log>\n");
-            (void) fclose(log_info->file);
-            log_info->file=(FILE *) NULL;
-          }
-      MagickFreeMemory(log_info->filename);
-      MagickFreeMemory(log_info->path);
-      MagickFreeMemory(log_info->format);
-      MagickFreeMemory(log_info);
-    }
-  log_info=(LogInfo *) NULL;
+  if (log_info.file != (FILE *) NULL)
+    if ((log_info.file != stdout) && (log_info.file != stderr))
+      {
+        (void) fprintf(log_info.file,"</log>\n");
+        (void) fclose(log_info.file);
+        log_info.file=(FILE *) NULL;
+      }
   log_configured=False;
 
   UnlockSemaphoreInfo(log_semaphore);
@@ -302,50 +291,37 @@ InitializeLogInfo(void)
   log_semaphore=AllocateSemaphoreInfo();
 
   /*
-    Allocate LogInfo
+    Initialize LogInfo
   */
-  assert(log_info == (LogInfo *) NULL);
-  log_info=MagickAllocateMemory(LogInfo *,sizeof(LogInfo));
-  if (log_info == (LogInfo *) NULL)
-    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-                      UnableToAllocateLogInfo);
-  (void) memset((void *) log_info,0,sizeof(LogInfo));
+  (void) memset(&log_info,0,sizeof(LogInfo));
 
   /*
     Lock for access (to make Coverity happy)
   */
+#if defined(__COVERITY__)
   LockSemaphoreInfo(log_semaphore);
+#endif /* defined(__COVERITY__) */
 
-  log_info->path=AcquireString("(default)");
-  log_info->filename=AcquireString("Magick-%d.log");
-  log_info->generations=3;
-  log_info->limit=2000;
-  log_info->format=AcquireString("%t %r %u %p %m/%f/%l/%d:\n  %e");
-  log_info->file=(FILE *) NULL;
-  log_info->generation=0;
-  log_info->count=0;
-  log_info->events=NoEventsMask;
+  (void) strlcpy(log_info.path,"(default)",sizeof(log_info.path));
+  (void) strlcpy(log_info.filename,"Magick-%d.log",sizeof(log_info.filename));
+  log_info.generations=3;
+  log_info.limit=2000;
+  (void) strlcpy(log_info.format,"%t %r %u %p %m/%f/%l/%d:\n  %e",
+                 sizeof(log_info.format));
+  log_info.file=(FILE *) NULL;
+  log_info.generation=0;
+  log_info.count=0;
+  log_info.events=NoEventsMask;
 #if defined(MSWINDOWS)
-  log_info->output_type=Win32EventlogOutput;
+  log_info.output_type=Win32EventlogOutput;
 #else
-  log_info->output_type=StderrOutput;
+  log_info.output_type=StderrOutput;
 #endif
-  GetTimerInfo(&log_info->timer);
-
-
-  /*
-    Verify subordinate allocations.
-  */
-  if ((log_info->path == (char *) NULL) ||
-      (log_info->filename == (char *) NULL) ||
-      (log_info->format == (char *) NULL))
-    {
-      UnlockSemaphoreInfo(log_semaphore);
-      MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-                        UnableToAllocateLogInfo);
-    }
+  GetTimerInfo(&log_info.timer);
 
+#if defined(__COVERITY__)
   UnlockSemaphoreInfo(log_semaphore);
+#endif /* defined(__COVERITY__) */
 
   /*
     Set initial logging flags using the value of MAGICK_DEBUG if it is
@@ -453,7 +429,7 @@ InitializeLogInfoPost(void)
 */
 MagickExport MagickBool IsEventLogging(void)
 {
-  return ((log_info) && (log_info->events != NoEventsMask));
+  return (log_info.events != NoEventsMask);
 }
 
 /*
@@ -519,7 +495,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
   if (!IsEventLogging())
     return(False);
 
-  if (log_info->events != AllEventsMask)
+  if (log_info.events != AllEventsMask)
     {
       unsigned int
         i;
@@ -540,7 +516,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
               if (((int) type >= eventmask_map[i].start_type) &&
                   ((int) type <= eventmask_map[i].end_type))
                 {
-                  if (((unsigned int) log_info->events) &
+                  if (((unsigned int) log_info.events) &
                       ((unsigned int) eventmask_map[i].mask))
                     {
                       enabled=True;
@@ -556,7 +532,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
               if ((((int) type % 100) >= eventmask_map[i].start_type) &&
                   (((int) type % 100) <= eventmask_map[i].end_type))
                 {
-                  if (((unsigned int) log_info->events) &
+                  if (((unsigned int) log_info.events) &
                       ((unsigned int) eventmask_map[i].mask))
                     {
                       enabled=True;
@@ -637,107 +613,107 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
 #endif
   seconds=time((time_t *) NULL);
   time_meridian=localtime(&seconds);
-  elapsed_time=GetElapsedTime(&log_info->timer);
-  user_time=GetUserTime(&log_info->timer);
-  (void) ContinueTimer((TimerInfo *) &log_info->timer);
+  elapsed_time=GetElapsedTime(&log_info.timer);
+  user_time=GetUserTime(&log_info.timer);
+  (void) ContinueTimer((TimerInfo *) &log_info.timer);
   FormatString(timestamp,"%04d%02d%02d%02d%02d%02d",time_meridian->tm_year+
     1900,time_meridian->tm_mon+1,time_meridian->tm_mday,
     time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
-  if (((unsigned int) log_info->output_type) & XMLFileOutput)
+  if (((unsigned int) log_info.output_type) & XMLFileOutput)
     {
       /*
         Log to a file in the XML format.
       */
-      log_info->count++;
-      if (log_info->count >= log_info->limit)
+      log_info.count++;
+      if (log_info.count >= log_info.limit)
         {
-          (void) fprintf(log_info->file,"</log>\n");
-          (void) fclose(log_info->file);
-          log_info->file=(FILE *) NULL;
-          log_info->count=0;
+          (void) fprintf(log_info.file,"</log>\n");
+          (void) fclose(log_info.file);
+          log_info.file=(FILE *) NULL;
+          log_info.count=0;
         }
-      if (log_info->file == (FILE *) NULL)
+      if (log_info.file == (FILE *) NULL)
         {
           char
             filename[MaxTextExtent];
 
-          (void) MagickSceneFileName(filename,log_info->filename,".%lu",
-                                     MagickFalse,log_info->generation);
-          log_info->file=fopen(filename,"w");
-          if (log_info->file == (FILE *) NULL)
+          (void) MagickSceneFileName(filename,log_info.filename,".%lu",
+                                     MagickFalse,log_info.generation);
+          log_info.file=fopen(filename,"w");
+          if (log_info.file == (FILE *) NULL)
             {
               UnlockSemaphoreInfo(log_semaphore);
               return(False);
             }
-          (void) fprintf(log_info->file,"<?xml version=\"1.0\"?>\n");
-          (void) fprintf(log_info->file,"<log>\n");
-          log_info->generation++;
-          if (log_info->generation >= log_info->generations)
-            log_info->generation=0;
+          (void) fprintf(log_info.file,"<?xml version=\"1.0\"?>\n");
+          (void) fprintf(log_info.file,"<log>\n");
+          log_info.generation++;
+          if (log_info.generation >= log_info.generations)
+            log_info.generation=0;
         }
-      (void) fprintf(log_info->file,"<record>\n");
-      (void) fprintf(log_info->file,"  <timestamp>%.1024s</timestamp>\n",
+      (void) fprintf(log_info.file,"<record>\n");
+      (void) fprintf(log_info.file,"  <timestamp>%.1024s</timestamp>\n",
         timestamp);
-      (void) fprintf(log_info->file,
+      (void) fprintf(log_info.file,
         "  <elapsed-time>%ld:%02ld</elapsed-time>\n",
         (long) (elapsed_time/60.0),(long) ceil(fmod(elapsed_time,60.0)));
-      (void) fprintf(log_info->file,"  <user-time>%0.3f</user-time>\n",
+      (void) fprintf(log_info.file,"  <user-time>%0.3f</user-time>\n",
         user_time);
-      (void) fprintf(log_info->file,"  <pid>%ld</pid>\n",(long) getpid());
-      (void) fprintf(log_info->file,"  <module>%.1024s</module>\n",srcname);
-      (void) fprintf(log_info->file,"  <function>%.1024s</function>\n",
+      (void) fprintf(log_info.file,"  <pid>%ld</pid>\n",(long) getpid());
+      (void) fprintf(log_info.file,"  <module>%.1024s</module>\n",srcname);
+      (void) fprintf(log_info.file,"  <function>%.1024s</function>\n",
         function);
-      (void) fprintf(log_info->file,"  <line>%lu</line>\n",line);
-      (void) fprintf(log_info->file,"  <domain>%.1024s</domain>\n",domain);
-      (void) fprintf(log_info->file,"  <severity>%.1024s</severity>\n",severity);
-      (void) fprintf(log_info->file,"  <event>%.1024s</event>\n",event);
-      (void) fprintf(log_info->file,"</record>\n");
-      (void) fflush(log_info->file);
+      (void) fprintf(log_info.file,"  <line>%lu</line>\n",line);
+      (void) fprintf(log_info.file,"  <domain>%.1024s</domain>\n",domain);
+      (void) fprintf(log_info.file,"  <severity>%.1024s</severity>\n",severity);
+      (void) fprintf(log_info.file,"  <event>%.1024s</event>\n",event);
+      (void) fprintf(log_info.file,"</record>\n");
+      (void) fflush(log_info.file);
       UnlockSemaphoreInfo(log_semaphore);
       return(True);
     }
-  if (((unsigned int) log_info->output_type) & TXTFileOutput)
+  if (((unsigned int) log_info.output_type) & TXTFileOutput)
     {
       /*
         Log to a file in the TXT format.
       */
-      log_info->count++;
-      if (log_info->count >= log_info->limit)
+      log_info.count++;
+      if (log_info.count >= log_info.limit)
         {
-          (void) fclose(log_info->file);
-          log_info->file=(FILE *) NULL;
-          log_info->count=0;
+          (void) fclose(log_info.file);
+          log_info.file=(FILE *) NULL;
+          log_info.count=0;
         }
-      if (log_info->file == (FILE *) NULL)
+      if (log_info.file == (FILE *) NULL)
         {
           char
             filename[MaxTextExtent];
 
-          (void) MagickSceneFileName(filename,log_info->filename,".%lu",
-                                     MagickFalse,log_info->generation);
-          log_info->file=fopen(filename,"w");
-          if (log_info->file == (FILE *) NULL)
+          (void) MagickSceneFileName(filename,log_info.filename,".%lu",
+                                     MagickFalse,log_info.generation);
+          log_info.file=fopen(filename,"w");
+          if (log_info.file == (FILE *) NULL)
             {
               UnlockSemaphoreInfo(log_semaphore);
               return(False);
             }
-          log_info->generation++;
-          if (log_info->generation >= log_info->generations)
-            log_info->generation=0;
+          log_info.generation++;
+          if (log_info.generation >= log_info.generations)
+            log_info.generation=0;
         }
-        (void) fprintf(log_info->file,
+        (void) fprintf(log_info.file,
                        "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu"
                        " %.1024s %.1024s %.1024s\n",
                        timestamp, (long) (elapsed_time/60.0),
                        (long) ceil(fmod(elapsed_time,60.0)),
                        user_time, (long) getpid(), srcname, function, line, domain,
                        severity, event);
-      (void) fflush(log_info->file);
+      (void) fflush(log_info.file);
       UnlockSemaphoreInfo(log_semaphore);
       return(True);
     }
 #if defined(MSWINDOWS)
-  if (log_info->output_type & Win32DebugOutput)
+  if (log_info.output_type & Win32DebugOutput)
     {
       char
         buffer[MaxTextExtent];
@@ -751,7 +727,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
                    domain, severity, event);
       OutputDebugString(buffer);
     }
-  if (log_info->output_type & Win32EventlogOutput)
+  if (log_info.output_type & Win32EventlogOutput)
     {
 #define LOGGING_ERROR_CODE 0
       char
@@ -779,8 +755,8 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         }
     }
 #endif
-  if ((((unsigned int) log_info->output_type) & StdoutOutput) ||
-      (((unsigned int) log_info->output_type) & StderrOutput))
+  if ((((unsigned int) log_info.output_type) & StdoutOutput) ||
+      (((unsigned int) log_info.output_type) & StderrOutput))
     {
       FILE
         *file;
@@ -788,9 +764,9 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         Log to stdout in a "human readable" format.
       */
       file = stdout;
-      if (((unsigned int) log_info->output_type) & StderrOutput)
+      if (((unsigned int) log_info.output_type) & StderrOutput)
         file = stderr;
-      for (p=log_info->format; *p != '\0'; p++)
+      for (p=log_info.format; *p != '\0'; p++)
       {
         /*
           Process formatting characters in text.
@@ -887,8 +863,8 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
       (void) fprintf(file,"\n");
       (void) fflush(file);
     }
-  if ((log_info->output_type & MethodOutput) &&
-      (log_info->method != (LogMethod) NULL))
+  if ((log_info.output_type & MethodOutput) &&
+      (log_info.method != (LogMethod) NULL))
     {
       char
         buffer[MaxTextExtent];
@@ -900,7 +876,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
                    (long) ceil(fmod(elapsed_time,60.0)),
                    user_time, (long) getpid(), srcname, function, line,
                    domain, severity, event);
-      log_info->method(type,buffer);
+      log_info.method(type,buffer);
     }
   UnlockSemaphoreInfo(log_semaphore);
   return(True);
@@ -964,13 +940,12 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
     *xml;
 
   size_t
-    length=0;
+    length=0,
+    token_max_length;
 
   MagickPassFail
     status=MagickPass;
 
-  assert(log_info != (LogInfo *) NULL);
-
   /*
     Read the log configure file.
   */
@@ -993,12 +968,13 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
   if (xml == (char *) NULL)
     return MagickFail;
   token=AllocateString(xml);
+  token_max_length=strlen(token);
   for (q=xml; *q != '\0'; )
   {
     /*
       Interpret Coder.
     */
-    GetToken(q,&q,token);
+    MagickGetToken(q,&q,token,token_max_length);
     if (*token == '\0')
       break;
     (void) strlcpy(keyword,token,MaxTextExtent);
@@ -1008,7 +984,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+          MagickGetToken(q,&q,token,token_max_length);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -1019,10 +995,10 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
         while ((*token != '>') && (*q != '\0'))
         {
           (void) strlcpy(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          MagickGetToken(q,&q,token,token_max_length);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          MagickGetToken(q,&q,token,token_max_length);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
@@ -1051,23 +1027,21 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       }
     if (LocaleCompare(keyword,"<magicklog>") == 0)
       {
-        (void) CloneString(&log_info->path,path);
+        (void) strlcpy(log_info.path,path,sizeof(log_info.path));
         continue;
       }
-    if (log_info == (LogInfo *) NULL)
-      continue;
-    GetToken(q,(char **) NULL,token);
+    MagickGetToken(q,(char **) NULL,token,token_max_length);
     if (*token != '=')
       continue;
-    GetToken(q,&q,token);
-    GetToken(q,&q,token);
+    MagickGetToken(q,&q,token,token_max_length);
+    MagickGetToken(q,&q,token,token_max_length);
     switch (*keyword)
     {
       case 'E':
       case 'e':
       {
         if (LocaleCompare((char *) keyword,"events") == 0)
-          log_info->events = (LogEventType) ((unsigned int) log_info->events |
+          log_info.events = (LogEventType) ((unsigned int) log_info.events |
                                              (unsigned int) ParseEvents(token));
         break;
       }
@@ -1076,12 +1050,12 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"filename") == 0)
           {
-            (void) CloneString(&log_info->filename,token);
+            (void) strlcpy(log_info.filename,token,sizeof(log_info.filename));
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
           {
-            (void) CloneString(&log_info->format,token);
+            (void) strlcpy(log_info.format,token,sizeof(log_info.format));
             break;
           }
         break;
@@ -1091,7 +1065,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"generations") == 0)
           {
-            log_info->generations=MagickAtoL(token);
+            log_info.generations=MagickAtoL(token);
             break;
           }
         break;
@@ -1101,7 +1075,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"limit") == 0)
           {
-            log_info->limit=MagickAtoL(token);
+            log_info.limit=MagickAtoL(token);
             break;
           }
         break;
@@ -1127,7 +1101,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
 
                   LockSemaphoreInfo(log_semaphore);
 
-                  log_info->output_type=output_map[i].mask;
+                  log_info.output_type=output_map[i].mask;
 
                   UnlockSemaphoreInfo(log_semaphore);
 
@@ -1186,10 +1160,10 @@ MagickExport unsigned long SetLogEventMask(const char *events)
   if (events != NULL)
     {
       event_flags=ParseEvents(events);
-      log_info->events=event_flags;
+      log_info.events=event_flags;
     }
 
-  event_flags=log_info->events;
+  event_flags=log_info.events;
 
   UnlockSemaphoreInfo(log_semaphore);
 
@@ -1227,7 +1201,7 @@ MagickExport void SetLogFormat(const char *format)
 {
   LockSemaphoreInfo(log_semaphore);
 
-  (void) CloneString(&log_info->format,format);
+  (void) strlcpy(log_info.format,format,sizeof(log_info.format));
 
   UnlockSemaphoreInfo(log_semaphore);
 }
@@ -1261,9 +1235,9 @@ MagickExport void SetLogMethod(LogMethod method)
 {
   LockSemaphoreInfo(log_semaphore);
 
-  log_info->output_type=(LogOutputType) (log_info->output_type |
+  log_info.output_type=(LogOutputType) (log_info.output_type |
     MethodOutput);
-  log_info->method=method;
+  log_info.method=method;
 
   UnlockSemaphoreInfo(log_semaphore);
 }
