@@ -90,7 +90,6 @@ StaticMagic[] =
   MAGIC("FITS", 0, "SIMPLE"),
   MAGIC("FPX", 0, "\320\317\021\340"),
   MAGIC("GIF", 0, "GIF8"),
-  MAGIC("GPLT", 0, "#!/usr/local/bin/gnuplot"),
   MAGIC("HDF", 1, "HDF"),
   MAGIC("HPGL", 0, "IN;"),
   MAGIC("HPGL", 0, "\033E\033"),
@@ -118,7 +117,8 @@ StaticMagic[] =
   MAGIC("PDF", 0, "%PDF-"),
   MAGIC("PFA", 0, "%!PS-AdobeFont-1.0"),
   MAGIC("PFB", 6, "%!PS-AdobeFont-1.0"),
-  MAGIC("PGX", 0, "\050\107\020\115\046"),
+  MAGIC("PGX", 0, "PG ML"),
+  MAGIC("PGX", 0, "PG LM"),
   MAGIC("PICT", 522, "\000\021\002\377\014\000"),
   MAGIC("PNG", 0, "\211PNG\r\n\032\n"),
   MAGIC("PBM", 0, "P1"),
