@@ -146,6 +146,7 @@ ModuleAliases[] =
   MODULEALIAS("J2C","JP2"),
   MODULEALIAS("JBG","JBIG"),
   MODULEALIAS("JNG","PNG"),
+  MODULEALIAS("JPC","JP2"),
   MODULEALIAS("JPG","JPEG"),
   MODULEALIAS("K","GRAY"),
   MODULEALIAS("K25","DCRAW"),
@@ -174,6 +175,7 @@ ModuleAliases[] =
   MODULEALIAS("PFA","TTF"),
   MODULEALIAS("PFB","TTF"),
   MODULEALIAS("PGM","PNM"),
+  MODULEALIAS("PGX","JP2"),
   MODULEALIAS("PICON","XPM"),
   MODULEALIAS("PM","XPM"),
   MODULEALIAS("PNG24","PNG"),
