@@ -17,7 +17,7 @@ PACKAGE_BUGREPORT='graphicsmagick-bugs@lists.sourceforge.net'
 
 # Package base version.  This is is the numeric version suffix applied to
 # PACKAGE_NAME (e.g. "1.2").
-PACKAGE_VERSION='1.3.23'
+PACKAGE_VERSION='1.3.26'
 
 #
 # Package name plus version string.
@@ -27,7 +27,7 @@ PACKAGE_STRING="$PACKAGE_NAME $PACKAGE_VERSION"
 #
 # Formal Package release date
 # Set to string "unreleased" if package is not a formal release.
-PACKAGE_RELEASE_DATE="2015-11-07"
+PACKAGE_RELEASE_DATE="2017-07-04"
 #PACKAGE_RELEASE_DATE="unreleased"
 
 #
@@ -50,7 +50,7 @@ fi
 #
 # Mercurial branch that this release is on.
 #
-HG_BRANCH_TAG=GraphicsMagick-1_3
+HG_BRANCH_TAG=default
 
 #
 # Libtool library revision control info
@@ -86,21 +86,21 @@ HG_BRANCH_TAG=GraphicsMagick-1_3
 #
 # Magick library versioning
 #
-MAGICK_LIBRARY_CURRENT=17
+MAGICK_LIBRARY_CURRENT=19
 
-MAGICK_LIBRARY_REVISION=1
-MAGICK_LIBRARY_AGE=14
+MAGICK_LIBRARY_REVISION=0
+MAGICK_LIBRARY_AGE=16
 
 #
 # Magick++ library versioning
 #
-MAGICK_PLUS_PLUS_LIBRARY_CURRENT=12
-MAGICK_PLUS_PLUS_LIBRARY_REVISION=1
-MAGICK_PLUS_PLUS_LIBRARY_AGE=0
+MAGICK_PLUS_PLUS_LIBRARY_CURRENT=14
+MAGICK_PLUS_PLUS_LIBRARY_REVISION=0
+MAGICK_PLUS_PLUS_LIBRARY_AGE=2
 
 #
 # Magick Wand library versioning
 #
-MAGICK_WAND_LIBRARY_CURRENT=9
-MAGICK_WAND_LIBRARY_REVISION=3
-MAGICK_WAND_LIBRARY_AGE=7
+MAGICK_WAND_LIBRARY_CURRENT=10
+MAGICK_WAND_LIBRARY_REVISION=0
+MAGICK_WAND_LIBRARY_AGE=8
